local AddonName, AddonTable = ...
local L = AddonTable.Localize
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")

playerName = UnitName("player")

function DRFrame_OnLoad()

	-- Create SavedVariables if they don't exist already
	if DRConfig == "" or DRConfig == nil then
		DRConfig = {
			["DeclineDuel"] = 'ON',
			["ShowChatMessage"] = 'ON',
			["ShowLoadMessage"] = 'ON',
			["OptionsWindow"] = 'HIDDEN',
			["AntiLag"] = 'OFF',
			["MaxLatency"] = '250',
			["FPSCheck"] = 'OFF',
			["MinFPS"] = '15',
			["ShowStats"] = 'YES',
			["HighLevelCheck"] = 'ENABLED',
			["StatsWindow"] = 'HIDDEN'
		}
	end
	
	if DRConfig.StatsWindow == "" or DRConfig.StatsWindow == nil then
		DRConfig.StatsWindow = 'HIDDEN'
	end
	
	-- This variable tracks the total number of blocked duels across all of your characters
	if GlobalDC == "" or GlobalDC == nil then
		GlobalDC = 0
	end
	
	-- This variable tracks the total number of blocked duels on a single character
	if DuelsDeclined == "" or DuelsDeclined == nil then
		DuelsDeclined = 0
	end
	
	DRFrame:RegisterEvent("ADDON_LOADED")
	DRFrame:RegisterEvent("VARIABLES_LOADED")
	DRFrame:RegisterEvent("DUEL_REQUESTED")
	
	-- already hidden by default, but just in case...
	DRFrame:Hide()
	
end

function ChallengeSentFrame_OnLoad()
	ChallengeSentFrame:RegisterEvent("CHAT_MSG_SYSTEM")
end

function ResSicknessFrame_OnLoad()
	ResSicknessFrame:RegisterEvent("DUEL_REQUESTED")
end

function FPSCheckFrame_OnLoad()
	FPSCheckFrame:RegisterEvent("DUEL_REQUESTED")
end

function LagCheckFrame_OnLoad()
	LagCheckFrame:RegisterEvent("DUEL_REQUESTED")
end

function AFKCheckFrame_OnLoad()
	AFKCheckFrame:RegisterEvent("DUEL_REQUESTED")
end

function DNDCheckFrame_OnLoad()
	DNDCheckFrame:RegisterEvent("DUEL_REQUESTED")
end

function DRPetBattleFrame_OnLoad()
	DRPetBattleFrame:RegisterEvent("PET_BATTLE_PVP_DUEL_REQUESTED")
end

function DRFrame_OnEvent(self, event, arg1)

if event == "ADDON_LOADED" and arg1 == "DuelResponse" then
	resetcheck()
	if DRConfig.ShowLoadMessage == "ON" then
		-- Show these messages in chat when the addon is loaded
		DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFF[" .. L["DuelResponse"] .."]|r v" .. GetAddOnMetadata("DuelResponse", "Version") .. L[" loaded successfully."])
	else
		return
	end
elseif event == "VARIABLES_LOADED" then
	
	DRConfig.OptionsWindow = 'HIDDEN'

	-- Check for new SavedVariables; create them if they're missing.
	if DRConfig.HighLevelCheck == "" or DRConfig.HighLevelCheck == nil then
		DRConfig.HighLevelCheck = 'ENABLED'
	end
	
	if DRConfig.OptionsWindow == "" or DRConfig.OptionsWindow == nil then
		DRConfig.OptionsWindow = 'HIDDEN'
	end
	
	if DRConfig.DeclinePB == "" or DRConfig.DeclinePB == nil then
		DRConfig.DeclinePB = 'OFF'
	end
	
	if DRConfig.AntiLag == "" or DRConfig.AntiLag == nil then
		DRConfig.AntiLag = 'NO'
	end
	
	if DRConfig.MaxLatency == "" or DRConfig.MaxLatency == nil then
		DRConfig.MaxLatency = '250'
	end
	
	if DRConfig.FPSCheck == "" or DRConfig.FPSCheck == nil then
		DRConfig.FPSCheck = 'OFF'
	end
	
	if DRConfig.MinFPS == "" or DRConfig.MinFPS == nil then
		DRConfig.MinFPS = '15'
	end
	
	if DRConfig.OptionsVis == "" or DRConfig.OptionsVis == nil then
		DRConfig.OptionsVis = '0.9'
	end
	
	if DRConfig.ShowStats == "" or DRConfig.ShowStats == nil then
		DRConfig.ShowStats = 'YES'
	end
elseif event == "DUEL_REQUESTED" then

	if DRConfig.DeclineDuel == "ON" and UnitIsAFK("player") == false and UnitIsDND("player") == false then
		-- If the player is blocking duels, holding down either Control or Alt will allow a request to pass through.
		if IsControlKeyDown() == true or IsAltKeyDown() == true then
			return
		elseif IsControlKeyDown() == false and IsAltKeyDown() == false then
			-- If the player is blocking duels, decline the request and increment the counters.
			CancelDuel()
			increment_counters()
			
			if IsAddOnLoaded("DuelResponse_Blacklist") == true then
				if has_value(DRBIgnore, arg1) == true then
					DRChatMessage(tostring(arg1), L["Blacklisted"])
				elseif has_value(DRBIgnore, arg1) == false then
					DRChatMessage(tostring(arg1), L["Refusing all duels"])
				end
			elseif IsAddOnLoaded("DuelResponse_Blacklist") == false then
				DRChatMessage(tostring(arg1), L["Refusing all duels"])
			end
						
			if IsAddOnLoaded("DuelResponse_Whisper") == true then
				if UnitIsAFK("player") == true then
					C_ChatInfo.SendAddonMessage("DRALLDUELSAFK",tostring(arg1),"WHISPER",playerName)
				elseif UnitIsAFK("player") == false then
					C_ChatInfo.SendAddonMessage("DRALLDUELS",tostring(arg1),"WHISPER",playerName)
				end
			end
		end
	else
		return
	end
end
end

function ResSicknessFrame_OnEvent(self, event, arg1)

if event == "DUEL_REQUESTED" then
	if DRConfig.DeclineDuel == "OFF" then
		-- Check to see if the player has Resurrection Sickness; decline the request if they do.
		if AuraUtil.FindAuraByName(L["Resurrection Sickness"], "player") then
					
			CancelDuel()
			increment_counters()
			
			if IsAddOnLoaded("DuelResponse_Blacklist") == true then
				if has_value(DRBIgnore, arg1) == true then
					DRChatMessage(tostring(arg1), "Blacklisted")
				elseif has_value(DRBIgnore, arg1) == false then
					DRChatMessage(tostring(arg1), L["Resurrection Sickness"])
				end
			elseif IsAddOnLoaded("DuelResponse_Blacklist") == false then
				DRChatMessage(tostring(arg1), L["Resurrection Sickness"])
			end
			
			if IsAddOnLoaded("DuelResponse_Whisper") == true then
				if UnitIsAFK("player") == true then
					C_ChatInfo.SendAddonMessage("DRRESSICKNESSAFK",tostring(arg1),"WHISPER",playerName) end
				elseif UnitIsAFK("player") == false then
					C_ChatInfo.SendAddonMessage("DRRESSICKNESS",tostring(arg1),"WHISPER",playerName) end
				end
		elseif not AuraUtil.FindAuraByName(L["Resurrection Sickness"], "player") then
			return
		end
	end
end

function FPSCheckFrame_OnEvent(self, event, arg1)

if event == "DUEL_REQUESTED" then
	if IsControlKeyDown() == true or IsAltKeyDown() == true then
		return
	else
		local framerate = GetFramerate()
		if ceil(framerate) <= tonumber(DRConfig.MinFPS) then
			if DRConfig.FPSCheck == "ON" then

				CancelDuel()
				increment_counters()
				
				if IsAddOnLoaded("DuelResponse_Blacklist") == true then
					if has_value(DRBIgnore, arg1) == true then
						DRChatMessage(tostring(arg1), L["Blacklisted"])
					elseif has_value(DRBIgnore, arg1) == false then
						DRChatMessage(tostring(arg1), L["Low FPS"])
					end
				elseif IsAddOnLoaded("DuelResponse_Blacklist") == false then
					DRChatMessage(tostring(arg1), L["Low FPS"])
				end
				
				if IsAddOnLoaded("DuelResponse_Whisper") == true then
					if UnitIsAFK("player") == true then
						C_ChatInfo.SendAddonMessage("DRLOWFPSAFK",tostring(arg1),"WHISPER",playerName)
					elseif UnitIsAFK("player") == false then
						C_ChatInfo.SendAddonMessage("DRLOWFRAMERATE",tostring(arg1),"WHISPER",playerName)
					end
				end
			elseif DRConfig.FPSCheck == "OFF" then
				return
			end
		end
	end
end

end

function LagCheckFrame_OnEvent(self, event, arg1)

if event == "DUEL_REQUESTED" then
	if IsControlKeyDown() == true or IsAltKeyDown() == true then
		return
	else
		local down, up, lagHome, lagWorld = GetNetStats()
		if lagHome >= ceil(tonumber(DRConfig.MaxLatency)) or lagWorld >= ceil(tonumber(DRConfig.MaxLatency)) then
			if DRConfig.AntiLag == "ON" then
			-- Decline the duel if the option is enabled.

			CancelDuel()
			increment_counters()
			
			if IsAddOnLoaded("DuelResponse_Blacklist") == true then
				if has_value(DRBIgnore, arg1) == true then
					DRChatMessage(tostring(arg1), L["Blacklisted"])
				elseif has_value(DRBIgnore, arg1) == false then
					DRChatMessage(tostring(arg1), L["High latency"])
				end
			elseif IsAddOnLoaded("DuelResponse_Blacklist") == false then
				DRChatMessage(tostring(arg1), L["High latency"])
			end
			
			if IsAddOnLoaded("DuelResponse_Whisper") == true then
				if UnitIsAFK("player") == true then
					C_ChatInfo.SendAddonMessage("DRHIGHLATENCYAFK",tostring(arg1),"WHISPER",playerName)
				elseif UnitIsAFK("player") == false then
					C_ChatInfo.SendAddonMessage("DRHIGHLATENCY",tostring(arg1),"WHISPER",playerName)
				end
			end		
		elseif DRConfig.AntiLag == "OFF" then
				return
			end
		end
	end
end

end

function AFKCheckFrame_OnEvent(self, event, arg1)

if event == "DUEL_REQUESTED" then
	if IsControlKeyDown() == true or IsAltKeyDown() == true then
		return
	else
		if DRConfig.DeclineDuel == "ON" then
			return
		else
			local down, up, lagHome, lagWorld = GetNetStats()
			local framerate = GetFramerate()
			
			if UnitIsAFK("player") == true and lagHome < ceil(tonumber(DRConfig.MaxLatency)) and lagWorld < ceil(tonumber(DRConfig.MaxLatency)) and ceil(framerate) > tonumber(DRConfig.MinFPS) and not AuraUtil.FindAuraByName(L["Resurrection Sickness"], "player") then
				-- If the player is not blocking duels, but is marked as AFK and has no other issues (such as lag or Res Sickness), decline the request.
				CancelDuel()
				increment_counters()
				
				if IsAddOnLoaded("DuelResponse_Blacklist") == true then
					if has_value(DRBIgnore, arg1) == true then
						DRChatMessage(tostring(arg1), L["Blacklisted"])
					elseif has_value(DRBIgnore, arg1) == false then
						DRChatMessage(tostring(arg1), L["AFK"])
					end
				elseif IsAddOnLoaded("DuelResponse_Blacklist") == false then
					DRChatMessage(tostring(arg1), L["AFK"])
				end
			
				if IsAddOnLoaded("DuelResponse_Whisper") == true then C_ChatInfo.SendAddonMessage("DRPLAYERAFK",tostring(arg1),"WHISPER",playerName) end
				
			elseif UnitIsAFK("player") == false then
				return
			end
		end
	end
end

end

function DNDCheckFrame_OnEvent(self, event, arg1)

if event == "DUEL_REQUESTED" then
	if IsControlKeyDown() == true or IsAltKeyDown() == true then
		return
	else
		if DRConfig.DeclineDuel == "ON" then
			return
		else
			if UnitIsDND("player") == true then
				-- If the player is not blocking duels, but is marked as Busy, decline the request.
				CancelDuel()
				increment_counters()
				
				if IsAddOnLoaded("DuelResponse_Blacklist") == true then
					if has_value(DRBIgnore, arg1) == true then
						DRChatMessage(tostring(arg1), L["Blacklisted"])
					elseif has_value(DRBIgnore, arg1) == false then
						DRChatMessage(tostring(arg1), L["Busy"])
					end
				elseif IsAddOnLoaded("DuelResponse_Blacklist") == false then
					DRChatMessage(tostring(arg1), L["Busy"])
				end
				
				if IsAddOnLoaded("DuelResponse_Whisper") == true then C_ChatInfo.SendAddonMessage("DRPLAYERDND",tostring(arg1),"WHISPER",playerName) end

			elseif UnitIsDND("player") == false then
				return
			end
		end
	end
end

end

function DRPetBattleFrame_OnEvent(self, event, arg1)

if event == "PET_BATTLE_PVP_DUEL_REQUESTED" then
	-- Automatically declines a pet battle request if the option is enabled.
	if DRConfig.DeclinePB == 'ON' then
		if IsControlKeyDown() == true or IsAltKeyDown() == true then
			-- Player is allowing a request through, so do nothing.
			return
		elseif IsControlKeyDown() == false and IsAltKeyDown() == false then
			local theirname, theirrealm = string.match(tostring(arg1), "(.*)%-(.*)")
			local myrealm = GetRealmName("player")
			-- Decline the request if neither button is being held down.
			
			if theirrealm == myrealm or theirrealm ~= myrealm then
				C_PetBattles.CancelPVPDuel()
				DRChatMessage(tostring(arg1), L["Pet Battle"])
			end
			
			if IsAddOnLoaded("DuelResponse_Whisper") == true then
				if UnitIsAFK("player") == true then
					C_ChatInfo.SendAddonMessage("DRPETBATTLEAFK",tostring(arg1),"WHISPER",playerName)
				elseif UnitIsAFK("player") == false then
					C_ChatInfo.SendAddonMessage("DRPETBATTLE",tostring(arg1),"WHISPER",playerName)
				end
			end
		end
	elseif DRConfig.DeclinePB == 'OFF' then
		return
	end
end

end

function ChallengeSentFrame_OnEvent(self, event, arg1)

if event == "CHAT_MSG_SYSTEM" then
	if arg1 == L["You have requested a duel."] then
		if UnitIsAFK("target") == true then
			CancelDuel()
			DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFF[" .. L["DuelResponse"] .."]|r " .. L["Duel cancelled; target is AFK."])
		elseif UnitIsAFK("target") == false then
			if DRConfig.HighLevelCheck == "ENABLED" then
				local mylevel = tonumber(UnitLevel("player"))
				local theirlevel = tonumber(UnitLevel("target"))
				local lvldifference = tonumber(theirlevel - mylevel)
		
				if lvldifference <= 5 then
					return
				else
					CancelDuel()
					DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFF[" .. L["DuelResponse"] .."]|r " .. L["Duel cancelled; target is a higher level than you."])
				end
			end
		end
	end
end

end

--|---------------------------|--
--|      OTHER FUNCTIONS      |--
--|---------------------------|--
	
function increment_counters()

-- Set up variables
local x = tonumber(GlobalDC)
local y = tonumber(DuelsDeclined)

-- Increment counters by 1
x = x + 1
y = y + 1

-- Save new variables
GlobalDC = x
DuelsDeclined = y

end

function resetcheck()

local x = tonumber(GlobalDC) -- number of duel requests blocked across all of your characters
local y = tonumber(DuelsDeclined) -- number of duel requests blocked on the character you are currently playing

-- Check to see if the all character counter has been reset to zero, and if the local counter is greater than zero.
-- If both of these conditions are true, then reset the local counter to zero.
if x == 0 and y > 0 then
	y = 0
	DuelsDeclined = y
end

end

function DRChatMessage(challenger, reason)

if DRConfig.ShowChatMessage == "OFF" then
	return
elseif DRConfig.ShowChatMessage == "ON" then
	if reason == L["Pet Battle"] then
		DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFF[" .. L["DuelResponse"] .."]|r " .. L["Pet Battle declined: "] .. challenger)
	elseif reason ~= L["Pet Battle"] then
		DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFF[" .. L["DuelResponse"] .."]|r " .. L["Duel declined: "] .. challenger .. " (" .. reason .. ")")
	end
end

end